@@ -1,5 +1,5 @@
 /* GSequencer - Advanced GTK Sequencer
- * Copyright (C) 2005-2015 Joël Krähemann
+ * Copyright (C) 2005-2018 Joël Krähemann
  *
  * This file is part of GSequencer.
  *
@@ -25,6 +25,9 @@
 
 #include <pthread.h>
 
+#include <ags/libags.h>
+
+#include <ags/audio/ags_sound_enums.h>
 #include <ags/audio/ags_port.h>
 #include <ags/audio/ags_recall_id.h>
 #include <ags/audio/ags_recall_dependency.h>
@@ -38,8 +41,8 @@
 
 #define AGS_RECALL_HANDLER(handler)    ((AgsRecallHandler *)(handler))
 
-#define AGS_RECALL_DEFAULT_VERSION "0.4.2"
-#define AGS_RECALL_DEFAULT_BUILD_ID "CEST 02-10-2014 19:36"
+#define AGS_RECALL_DEFAULT_VERSION "2.0.0"
+#define AGS_RECALL_DEFAULT_BUILD_ID "Tue Feb  6 14:27:35 UTC 2018"
 
 typedef struct _AgsRecall AgsRecall;
 typedef struct _AgsRecallClass AgsRecallClass;
@@ -47,32 +50,13 @@ typedef struct _AgsRecallHandler AgsRecallHandler;
 
 /**
  * AgsRecallFlags:
- * @AGS_RECALL_ADDED_TO_REGISTRY: recall has been added to registry
+ * @AGS_RECALL_ADDED_TO_REGISTRY: the recall was added to registry, see #AgsConnectable::add_to_registry()
  * @AGS_RECALL_CONNECTED: indicates the port was connected by calling #AgsConnectable::connect()
- * @AGS_RECALL_RUN_INITIALIZED: run initialized
  * @AGS_RECALL_TEMPLATE: is template
- * @AGS_RECALL_PLAYBACK: does playback
- * @AGS_RECALL_SEQUENCER: does sequencer
- * @AGS_RECALL_NOTATION: does notation
  * @AGS_RECALL_DEFAULT_TEMPLATE: 
- * @AGS_RECALL_DYNAMIC_CONNECTED: dynamic connected
- * @AGS_RECALL_INPUT_ORIENTATED: input orientated
- * @AGS_RECALL_OUTPUT_ORIENTATED: output orientated
- * @AGS_RECALL_PERSISTENT: persistent processing
- * @AGS_RECALL_INITIAL_RUN: initial call to #AgsRecall::run()
- * @AGS_RECALL_TERMINATING: is terminating
- * @AGS_RECALL_DONE: is done
- * @AGS_RECALL_REMOVE: is remove
- * @AGS_RECALL_HIDE: is hidden
- * @AGS_RECALL_PROPAGATE_DONE: propagate done 
- * @AGS_RECALL_PERSISTENT_PLAYBACK: persistent playback
- * @AGS_RECALL_PERSISTENT_SEQUENCER: persistent sequencer
- * @AGS_RECALL_PERSISTENT_NOTATION: persistent notation
- * @AGS_RECALL_SKIP_DEPENDENCIES: skip dependencies
- * @AGS_RECALL_BULK_MODE: the recall operates on bulk mode
  * @AGS_RECALL_HAS_OUTPUT_PORT: has output port
- * @AGS_RECALL_RUN_FIRST: scheduled for run-first
- * @AGS_RECALL_RUN_LAST: scheduled for run-last
+ * @AGS_RECALL_BYPASS: don't apply effect processing
+ * @AGS_RECALL_INITIAL_RUN: initial run, first attack to audio data
  * 
  * Enum values to control the behavior or indicate internal state of #AgsRecall by
  * enable/disable as flags.
@@ -80,31 +64,11 @@ typedef struct _AgsRecallHandler AgsRecallHandler;
 typedef enum{
   AGS_RECALL_ADDED_TO_REGISTRY     = 1,
   AGS_RECALL_CONNECTED             = 1 <<  1,
-  AGS_RECALL_RUN_INITIALIZED       = 1 <<  2, //TODO:JK: rename to AGS_RECALL_RUN_CONNECTED
-  AGS_RECALL_TEMPLATE              = 1 <<  3,
-  AGS_RECALL_PLAYBACK              = 1 <<  4,
-  AGS_RECALL_SEQUENCER             = 1 <<  5,
-  AGS_RECALL_NOTATION              = 1 <<  6,
-  AGS_RECALL_DEFAULT_TEMPLATE      = 1 <<  7,
-  AGS_RECALL_DYNAMIC_CONNECTED     = 1 <<  8,
-  AGS_RECALL_INPUT_ORIENTATED      = 1 <<  9,
-  AGS_RECALL_OUTPUT_ORIENTATED     = 1 << 10,
-  AGS_RECALL_PERSISTENT            = 1 << 11,
-  AGS_RECALL_INITIAL_RUN           = 1 << 12,
-  AGS_RECALL_TERMINATING           = 1 << 13,
-  AGS_RECALL_DONE                  = 1 << 14,
-  AGS_RECALL_REMOVE                = 1 << 15,
-  AGS_RECALL_HIDE                  = 1 << 16,
-  AGS_RECALL_PROPAGATE_DONE        = 1 << 17, // see ags_recall_real_remove
-  AGS_RECALL_PERSISTENT_PLAYBACK   = 1 << 18,
-  AGS_RECALL_PERSISTENT_SEQUENCER  = 1 << 19,
-  AGS_RECALL_PERSISTENT_NOTATION   = 1 << 20,
-  AGS_RECALL_SKIP_DEPENDENCIES     = 1 << 21,
-  AGS_RECALL_BULK_MODE             = 1 << 22,
-  AGS_RECALL_HAS_OUTPUT_PORT       = 1 << 23,
-  AGS_RECALL_RUN_FIRST             = 1 << 24,
-  AGS_RECALL_RUN_LAST              = 1 << 25,
-  AGS_RECALL_PACKED                = 1 << 26,
+  AGS_RECALL_TEMPLATE              = 1 <<  2,
+  AGS_RECALL_DEFAULT_TEMPLATE      = 1 <<  3,
+  AGS_RECALL_HAS_OUTPUT_PORT       = 1 <<  4,
+  AGS_RECALL_BYPASS                = 1 <<  5,
+  AGS_RECALL_INITIAL_RUN           = 1 <<  6,
 }AgsRecallFlags;
 
 /**
@@ -116,6 +80,7 @@ typedef enum{
  * @AGS_RECALL_NOTIFY_CHANNEL_RUN: notifiy dependency channel run
  * @AGS_RECALL_NOTIFY_RECALL: notifiy dependency recall
  * 
+ * Modes to notify of dependencies.
  */
 typedef enum{
   AGS_RECALL_NOTIFY_RUN,
@@ -131,71 +96,94 @@ struct _AgsRecall
   GObject object;
 
   guint flags;
-  gboolean rt_safe;
+  guint ability_flags;
+  guint behaviour_flags;
+  gint sound_scope;
+  guint staging_flags;
+  guint state_flags;
   
-  pthread_mutexattr_t *obj_mutexattr;
+  //  gboolean rt_safe; note replace by globals
+
   pthread_mutex_t *obj_mutex;
+  pthread_mutexattr_t *obj_mutexattr;
+  
+  AgsUUID *uuid;
 
   gchar *version;
   gchar *build_id;
 
-  gchar *effect;
   gchar *name;
 
+  gchar *filename;
+  gchar *effect;
+  guint effect_index;
+  
   gchar *xml_type;
   
-  GObject *soundcard;
-  GObject *container; // see AgsRecallContainer
+  GObject *recall_container;
 
-  GList *dependencies;
+  GObject *output_soundcard;
+  gint output_soundcard_channel;
+  
+  GObject *input_soundcard;
+  gint input_soundcard_channel;
+  
+  guint samplerate;
+  guint buffer_size;
+  guint format;
 
-  AgsRecallID *recall_id;
+  guint pad;
+  guint audio_channel;
 
-  pthread_mutexattr_t *children_attr;
-  pthread_mutex_t *children_mutex;
-
-  AgsRecall *parent;
-  GList *children;
-
-  GType child_type;
-  GParameter *child_parameters;
-  guint n_params;
+  guint line;
 
   GList *port;
   GList *automation_port;
   
-  GList *handlers;
+  AgsRecallID *recall_id;
+
+  GList *recall_dependency;
+  GList *recall_handler;  
+
+  AgsRecall *parent;
+  
+  GType child_type;
+  
+  guint n_child_params;
+  gchar **child_parameter_name;
+  GValue *child_value;
+
+  GList *children;  
 };
 
 struct _AgsRecallClass
 {
   GObjectClass object;
 
-  void (*load_automation)(AgsRecall *recall,
-			  GList *automation_port);
-  void (*unload_automation)(AgsRecall *recall);
+  void (*resolve_dependency)(AgsRecall *recall);
+  void (*check_rt_data)(AgsRecall *recall);
   
-  void (*resolve_dependencies)(AgsRecall *recall);
-
   void (*run_init_pre)(AgsRecall *recall);
   void (*run_init_inter)(AgsRecall *recall);
   void (*run_init_post)(AgsRecall *recall);
-  void (*check_rt_stream)(AgsRecall *recall);
   
+  void (*feed_input_queue)(AgsRecall *recall);
   void (*automate)(AgsRecall *recall);
+  
   void (*run_pre)(AgsRecall *recall);
   void (*run_inter)(AgsRecall *recall);
   void (*run_post)(AgsRecall *recall);
+
+  void (*do_feedback)(AgsRecall *recall);
+  void (*feed_output_queue)(AgsRecall *recall);
   
   void (*stop_persistent)(AgsRecall *recall);
-  void (*done)(AgsRecall *recall);
-
   void (*cancel)(AgsRecall *recall);
-  void (*remove)(AgsRecall *recall);
+  void (*done)(AgsRecall *recall);
 
   AgsRecall* (*duplicate)(AgsRecall *recall,
 			  AgsRecallID *recall_id,
-			  guint *n_params, GParameter *params); // if a sequencer is linked with a sequencer the AgsRecall's with the flag AGS_RECALL_SOURCE must be duplicated
+			  guint *n_params, gchar **parameter_name, GValue *value);
 
   void (*notify_dependency)(AgsRecall *recall, guint dependency, gboolean increase);
 
@@ -216,83 +204,134 @@ struct _AgsRecallHandler
   const gchar *signal_name;
   GCallback callback;
   GObject *data;
-  gulong handler;
 };
 
 GType ags_recall_get_type();
 
 pthread_mutex_t* ags_recall_get_class_mutex();
 
+gboolean ags_recall_global_get_children_lock_free();
+gboolean ags_recall_global_get_omit_event();
+gboolean ags_recall_global_get_performance_mode();
+gboolean ags_recall_global_get_rt_safe();
+
+gboolean ags_recall_test_flags(AgsRecall *recall, guint flags);
 void ags_recall_set_flags(AgsRecall *recall, guint flags);
+void ags_recall_unset_flags(AgsRecall *recall, guint flags);
 
-void ags_recall_load_automation(AgsRecall *recall,
-				GList *automation_port);
-void ags_recall_unload_automation(AgsRecall *recall);
+/* ability flags */
+gboolean ags_recall_test_ability_flags(AgsRecall *recall, guint ability_flags);
+void ags_recall_set_ability_flags(AgsRecall *recall, guint ability_flags);
+void ags_recall_unset_ability_flags(AgsRecall *recall, guint ability_flags);
 
-void ags_recall_resolve_dependencies(AgsRecall *recall);
-void ags_recall_child_added(AgsRecall *parent, AgsRecall *child);
+gboolean ags_recall_check_ability_flags(AgsRecall *recall, guint ability_flags);
+gboolean ags_recall_match_ability_flags_to_scope(AgsRecall *recall, gint sound_scope);
 
-void ags_recall_run_init_pre(AgsRecall *recall);
-void ags_recall_run_init_inter(AgsRecall *recall);
-void ags_recall_run_init_post(AgsRecall *recall);
-void ags_recall_check_rt_stream(AgsRecall *recall);
+/* behaviour flags */
+gboolean ags_recall_test_behaviour_flags(AgsRecall *recall, guint behaviour_flags);
+void ags_recall_set_behaviour_flags(AgsRecall *recall, guint behaviour_flags);
+void ags_recall_unset_behaviour_flags(AgsRecall *recall, guint behaviour_flags);
 
-void ags_recall_automate(AgsRecall *recall);
-void ags_recall_run_pre(AgsRecall *recall);
-void ags_recall_run_inter(AgsRecall *recall);
-void ags_recall_run_post(AgsRecall *recall);
+gboolean ags_recall_check_behaviour_flags(AgsRecall *recall, guint behaviour_flags);
 
-void ags_recall_stop_persistent(AgsRecall *recall);
-void ags_recall_done(AgsRecall *recall);
+/* scope */
+void ags_recall_set_sound_scope(AgsRecall *recall, gint sound_scope);
+gint ags_recall_get_sound_scope(AgsRecall *recall);
 
-void ags_recall_cancel(AgsRecall *recall);
-void ags_recall_remove(AgsRecall *recall);
+gboolean ags_recall_check_sound_scope(AgsRecall *recall, gint sound_scope);
 
-gboolean ags_recall_is_done(GList *recalls, GObject *recycling_context);
+/* staging flags */
+gboolean ags_recall_test_staging_flags(AgsRecall *recall, guint behaviour_flags);
+void ags_recall_set_staging_flags(AgsRecall *recall, guint staging_flags);
+void ags_recall_unset_staging_flags(AgsRecall *recall, guint staging_flags);
 
-AgsRecall* ags_recall_duplicate(AgsRecall *recall,
-				AgsRecallID *recall_id);
+gboolean ags_recall_check_staging_flags(AgsRecall *recall, guint staging_flags);
 
+/* state flags */
+gboolean ags_recall_test_state_flags(AgsRecall *recall, guint behaviour_flags);
+void ags_recall_set_state_flags(AgsRecall *recall, guint state_flags);
+void ags_recall_unset_state_flags(AgsRecall *recall, guint state_flags);
+
+gboolean ags_recall_check_state_flags(AgsRecall *recall, guint state_flags);
+
+/* children */
 void ags_recall_set_recall_id(AgsRecall *recall, AgsRecallID *recall_id);
-void ags_recall_set_soundcard_recursive(AgsRecall *recall, GObject *soundcard);
 
-void ags_recall_notify_dependency(AgsRecall *recall, guint flags, gint count);
+void ags_recall_add_recall_dependency(AgsRecall *recall, AgsRecallDependency *recall_dependency);
+void ags_recall_remove_recall_dependency(AgsRecall *recall, AgsRecallDependency *recall_dependency);
 
-void ags_recall_add_dependency(AgsRecall *recall, AgsRecallDependency *recall_dependency);
-void ags_recall_remove_dependency(AgsRecall *recall, AgsRecall *dependency);
-GList* ags_recall_get_dependencies(AgsRecall *recall);
-
-void ags_recall_remove_child(AgsRecall *recall, AgsRecall *child);
 void ags_recall_add_child(AgsRecall *parent, AgsRecall *child);
-GList* ags_recall_get_children(AgsRecall *recall);
-
-GList* ags_recall_get_by_effect(GList *list, gchar *filename, gchar *effect);
-GList* ags_recall_find_recall_id_with_effect(GList *list, AgsRecallID *recall_id, gchar *filename, gchar *effect);
-
-GList* ags_recall_find_type(GList *recall_i, GType type);
-GList* ags_recall_find_template(GList *recall_i);
-GList* ags_recall_template_find_type(GList *recall_i, GType type);
-GList* ags_recall_template_find_all_type(GList *recall_i, ...);
-GList* ags_recall_find_type_with_recycling_context(GList *recall_i, GType type, GObject *recycling_context);
-GList* ags_recall_find_recycling_context(GList *recall_i, GObject *recycling_context);
-GList* ags_recall_find_provider(GList *recall, GObject *provider);
-GList* ags_recall_template_find_provider(GList *recall, GObject *provider);
-GList* ags_recall_find_provider_with_recycling_context(GList *recall_i, GObject *provider, GObject *recycling_context);
-
-void ags_recall_run_init(AgsRecall *recall, guint stage);
+void ags_recall_remove_child(AgsRecall *recall, AgsRecall *child);
 
+void ags_recall_handler_free(AgsRecallHandler *recall_handler);
 AgsRecallHandler* ags_recall_handler_alloc(const gchar *signal_name,
 					   GCallback callback,
 					   GObject *data);
 
-void ags_recall_add_handler(AgsRecall *recall,
-			    AgsRecallHandler *recall_handler);
-void ags_recall_remove_handler(AgsRecall *recall,
-			       AgsRecallHandler *recall_handler);
+void ags_recall_add_recall_handler(AgsRecall *recall,
+				   AgsRecallHandler *recall_handler);
+void ags_recall_remove_recall_handler(AgsRecall *recall,
+				      AgsRecallHandler *recall_handler);
 
+/* soundcard */
+void ags_recall_set_output_soundcard(AgsRecall *recall, GObject *output_soundcard);
+void ags_recall_set_input_soundcard(AgsRecall *recall, GObject *input_soundcard);
+
+/* presets */
+void ags_recall_set_samplerate(AgsRecall *recall, guint samplerate);
+void ags_recall_set_buffer_size(AgsRecall *recall, guint buffer_size);
+void ags_recall_set_format(AgsRecall *recall, guint format);
+
+/* events */
+void ags_recall_resolve_dependency(AgsRecall *recall);
+void ags_recall_check_rt_data(AgsRecall *recall);
+
+void ags_recall_run_init_pre(AgsRecall *recall);
+void ags_recall_run_init_inter(AgsRecall *recall);
+void ags_recall_run_init_post(AgsRecall *recall);
+
+void ags_recall_feed_input_queue(AgsRecall *recall);
+void ags_recall_automate(AgsRecall *recall);
+
+void ags_recall_run_pre(AgsRecall *recall);
+void ags_recall_run_inter(AgsRecall *recall);
+void ags_recall_run_post(AgsRecall *recall);
+
+void ags_recall_do_feedback(AgsRecall *recall);
+void ags_recall_feed_output_queue(AgsRecall *recall);
+
+void ags_recall_stop_persistent(AgsRecall *recall);
+void ags_recall_cancel(AgsRecall *recall);
+void ags_recall_done(AgsRecall *recall);
+
+AgsRecall* ags_recall_duplicate(AgsRecall *recall,
+				AgsRecallID *recall_id,
+				guint *n_params, gchar **parameter_name, GValue *value);
+
+void ags_recall_notify_dependency(AgsRecall *recall, guint dependency, gboolean increase);
+void ags_recall_child_added(AgsRecall *parent, AgsRecall *child);
+
+/* query */
+gboolean ags_recall_is_done(GList *recalls, GObject *recycling_context);
+
+GList* ags_recall_get_by_effect(GList *list, gchar *filename, gchar *effect);
+GList* ags_recall_find_recall_id_with_effect(GList *list, AgsRecallID *recall_id, gchar *filename, gchar *effect);
+
+GList* ags_recall_find_type(GList *recall, GType type);
+GList* ags_recall_find_template(GList *recall);
+GList* ags_recall_template_find_type(GList *recall, GType type);
+GList* ags_recall_template_find_all_type(GList *recall, ...);
+GList* ags_recall_find_type_with_recycling_context(GList *recall, GType type, GObject *recycling_context);
+GList* ags_recall_find_recycling_context(GList *recall, GObject *recycling_context);
+GList* ags_recall_find_provider(GList *recall, GObject *provider);
+GList* ags_recall_template_find_provider(GList *recall, GObject *provider);
+GList* ags_recall_find_provider_with_recycling_context(GList *recall, GObject *provider, GObject *recycling_context);
+
+/* control */
 void ags_recall_lock_port(AgsRecall *recall);
 void ags_recall_unlock_port(AgsRecall *recall);
 
+/* instantiate */
 AgsRecall* ags_recall_new();
 
 #endif /*__AGS_RECALL_H__*/
